/*  %kw # %v    %n    %d    %t # */
/* Version # 2    META2ASM.C    9-Dec-93    11:54:48 # */
/* ***********************************************************
   METAGRAPHICS SOFTWARE CORPORATION (c) 1988      
   FILE: META2ASM.C                                
   Converts fonts from MetaWINDOW bitmap font files to
   1) MASM assembly language
   2) TI 34010 assembly language
************************************************************** */

#define make_exe    1         /* define for stand alone program */
/* #define debug       1       */

#include "stdio.h"
#include "stdlib.h"
#include "ctype.h"
#include "string.h"
#include "conio.h"

#include "metawndo.h"      /* master MetaWINDOW include file */


#define OK                    0
#define ERROR                 -1
#define ER_INP_NAME           -10
#define ER_OUT_NAME           -11
#define ER_NOMEM              -12
#define ER_FORMAT             -13
#define ER_WRITE              -14

#define MAX_LINE              255
#define IN_LINE               1

#define MASM                  1
#define TI                    2

int
meta2asm(inname, outname, style, inform )
char *inname, *outname;
int style;
void (*inform)( int, void * );

/*
Convert file name 'inname' from metawindow font format to either:
style = 1 = IBM MASM assembly
style = 2 = TI 34010 assembly
and write to file name 'outname'. Use function at 'inform' to 
report about progress, or NULL if no informing necessary.
Returns OK if ok, error code if had problems.
*/
{
     FILE  *output;
     dirRec dir_info;
     int i;
     char *dash, file_date[20], font_name[20], *p, *comment;
     fontRcd *fnt;


     void write_data( FILE *, unsigned char *, unsigned int, int, void(*)(int, void *) );

     /* get the input file date and size */
     i = FileQuery( inname, &dir_info, 1 );
     if( i == 0 )
          return ER_INP_NAME;

     /* convert date to string */
     sprintf( file_date, "%d/%d/%d", (dir_info.fileDate >> 5) & 0x0F,
                                     dir_info.fileDate & 0x1F,
                                    ( (dir_info.fileDate >> 9) & 0x7F ) + 80 );

     /* add extra to file size to avoid bug when buffer size = file size */
     dir_info.fileSize += 10L;

     /* open output file */
     output = fopen( outname, "w" );
     if( output == NULL )
          return ER_OUT_NAME;

     /* get input file into memory */
     fnt = malloc( ( unsigned int ) dir_info.fileSize );
     if( fnt == NULL )
          return ER_NOMEM;

     i = FileLoad( inname, (char *) fnt, ( unsigned int ) dir_info.fileSize );
     if( i <= 0 ) {
          free( fnt );
          return ER_INP_NAME;
     }

     /* check it for metawindows format */
/*   if( strncmp( fnt->fontSign, "METAFONT", 8 ) ) {    */
     if( ( fnt->fontVer & 0x0F ) != 2 )  {
          free( fnt );
          return ER_FORMAT;
     }

     /* make a usable global name from the font base name */
     strncpy( font_name, fnt->fontBaseName, 19 );
     font_name[19] = '\0';
     p = font_name;
     while( *p ) {
          if( !isalnum( *p ) )
               *p = '_';
          p++;
     }
     

     /* print out an information header */
     dash = "------------------------------------------------------------";

     switch( style ) {
      
          case TI:
               comment = ";";
               fprintf( output, "%s%s\n", comment, dash );
               fprintf( output, "%sMetagraphics TMS34010 font file\n", comment );
               break;

          case MASM:
          default:
               style = MASM;
               comment = ";";
               fprintf( output, "%s%s\n", comment, dash );
               fprintf( output, "%sMetagraphics IBM MASM font file\n", comment );
               fprintf( output, "%sNOTE: this file must be edited to include macro definitions\n", comment );
               fprintf( output, "%sfor .DATA and ENDDS or those lines must be changed to the\n", comment );
               fprintf( output, "%sspecific segment names used by the application\n", comment );
               break;
     }

     fprintf( output, "%sConverted from file %s dated %s\n", comment, inname, file_date );
     fprintf( output, "%s%s\n", comment, dash );
     fprintf( output, "%sFont Description:\n", comment );
     fprintf( output, "%sFont format version = %d.%d\n", comment, fnt->fontVer & 0x0F, fnt->fontVer >> 4 );
     fprintf( output, "%sName = %s\n", comment, fnt->fontBaseName );
     fprintf( output, "%sFixed height = %d\n", comment, fnt->chHeight );
     fprintf( output, "%sFixed width = %d\n", comment, fnt->chWidth );
     fprintf( output, "%sPoint size = %d\n", comment, fnt->fontPtSize );
     fprintf( output, "%s%s\n", comment, dash );

     if( style == MASM )
          fprintf( output, "\t.DATA\n" );
     
     fprintf( output, "\n%sDeclare global font name\n", comment );

     if( style == MASM ) {
          fprintf( output, "\tpublic\t_%s\n\n", font_name );
          fprintf( output, "_%s\tlabel\tbyte\n", font_name );
     }
     if( style == TI ) {
          fprintf( output, "\t.globl\t_%s\n\n", font_name );
          fprintf( output, "_%s:\n", font_name );
     }

     write_data( output, (unsigned char *) fnt, ( unsigned int) dir_info.fileSize, style, inform );

     if( style == MASM ) {
          fprintf( output, "\n\tENDS\n" );
          fprintf( output, "\n\tend\n" );
     }

     if( style == TI )
          fprintf( output, "\n\t.end\n" );

     /* close files */
     fclose( output );

     /* free memory */
     free( fnt );

     return OK;

}

#define BYTES_LINE  10

void
write_data( output, data, len, style, inform )
FILE *output;
unsigned char *data;
unsigned int len;
int style;
void (*inform)( int, void * );
/*
Write 'len' bytes of data at 'data' to file 'output' as either:
style = 1 = MASM .asm lines
style = 2 = TI .asm lines
*/
{
     int new_line, byte_num, line_num;
     char *p, buf[80];

     new_line = True;
     line_num = byte_num = 0;

     while( len ) {

          if( new_line ) {

               if( inform )
                    (*inform)( IN_LINE, &line_num );

               fprintf( output, "\n" );

               if( style == TI )
                    fprintf( output,"\t.byte\t");
               if( style == MASM )
                    fprintf( output,"\tdb\t");

               new_line = False;
               byte_num = 0;
               line_num++;
          }

          sprintf( buf, "%03xh", *data++ );
          p = buf;
          while( *p ) {
               *p = toupper( *p );
               p++;
          }
          fprintf( output, buf );

          byte_num++;
          len--;

          if( ( byte_num < BYTES_LINE ) && len )
               fprintf( output, ", " );
          else
               new_line = True;

     }
}


#ifdef make_exe

void
main (argc, argv)
int argc;
char *argv[];
{
     int i, style; 
     char *s, buf[80], *version, iname[15], oname[15];

     int meta2asm( char *, char *, int, void (*)( int, void * ) );
     int get_args( int, char *[], int * );
     void extend( char *, char *, int );
     void print_me( int, void * );

     version = "BETA 1.2";

     /* default output style is ibm MASM */
     style = MASM;

     if( get_args( argc, argv, &style ) ) {
          printf("Usage - META2ASM inname outname [-t -i]\n");
          printf("Converts Metawindow font files to either:\n");
          printf("-i     IBM MASM assembly language (default)\n");
          printf("-t     TI TMS34010 assembly language\n");
          printf("The file type .FNT or .ASM is assumed if not present.\n");
          printf("Version %s\n", version );
          exit(1);
     }


     i = InitGraphics( CGA640x200 );
     if( i ) {
          printf("Metawindow driver not loaded!\n");
          exit(1);
     }

     strncpy( iname, argv[1], 14 );
     iname[14] = '\0';
     extend( iname, ".fnt", False );
     strncpy( oname, argv[2], 14 );
     oname[14] = '\0';
     extend( oname, ".asm", False );

     printf("Output file style is ");
     if( style == TI )
          printf("TI TMS34010\n");
     if( style == MASM )
          printf("IBM MASM\n");

     i = meta2asm( iname, oname, style, print_me );

     printf("\n");


     switch( i ) {

          case OK:
               s = "Conversion complete";
               break;

          case ER_INP_NAME:
               sprintf( buf,"Can't open input file name: %s", iname );
               s = buf;
               break;

          case ER_FORMAT:
               s = "Input file is not in proper format";
               break;

          case ER_NOMEM:
               s = "Not enough memory to allocate required workspace";
               break;

          case ER_OUT_NAME:
               sprintf( buf, "Can't open output file name: %s", oname );
               s = buf;
               break;

          case ER_WRITE:
               s = "Error writing file";
               break;

          default:
               sprintf( buf, "Internal error #%d", i);
               s = buf;
               break;
     }

     if( i != OK )
          printf("There has been an error.\n");

     printf("%s\n", s);

#ifdef debug
     j = QueryError();
     printf("Query error = %d/%d\n", j >> 7, j & 127 );
     printf("Return value = %d\n", i);
#endif

     StopGraphics();
}

int
get_args( argc, argv, style )
int argc;
char *argv[];
int *style;
/*
Get command line arguments and set style type.
Returns OK or ERROR.
*/
{
     char c;

     /* must have at least two file names */
     if( argc < 3 )
          return ERROR;

     /* scan for switches */          
     while( --argc > 0 ) {
          
          if( ( *argv[argc] == '-' ) || ( *argv[argc] == '/' ) ) {
               
               c = toupper( *( argv[argc] + 1 ) );

               switch( c ) {

                    case 'T':
                         *style = TI;
                         break;

                    case 'I':
                         *style = MASM;
                         break;

                    default:
                         return ERROR;

               }

          }

     }

     return OK;
}

void
extend( name, ext, force )
char *name, *ext;
int force;
/*
Extend filename 'name' to include extention 'ext'. If force is true,
force it to have the 'ext' extention, if false, only put it on if one
doesn't already exist.
Note: 'name' buffer must be long enough to hold the added extention
      'ext' must include the '.'
*/
{
     char *p;

     p = name;
     while( *p && *p != '.' )
          p++;

     if( (*p != '.') || force )
          strcpy( p, ext );

}


void
print_me( code, var )
int code;
void *var;
/*
Inform user of actions occuring in conversion functions.
Code is an indicator of the action to inform, num is a gerneral purpose
parameter.
*/
{

     int *num;

     num = ( int *) var;

     switch( code ) {

          case IN_LINE:
               printf("Line %04d \r", *num );
               break;

          default:
               break;
     }

}
#endif /* make_exe */

